% This function is used to implement the CG iterative method
% with a preconditioner
%
% x=prec_cg(a,el,f,reps);
%
% programmer: C. Vuik, TU Delft, The Netherlands
% e-mail    : c.vuik@tudelft.nl
% date      : 02-02-2023
% course    : Computational Fluid Dynamics II
%             a PhD course from the JM Burgerscentrum
%

function x=prec_cg(a,el,f,reps);
close all
clear x n1 n2 n m r
format long e
r = f;
disp(' ')
disp(' ')
disp('     iteration                 norm of the residual')
disp('     ---------                 --------------------')
disp(' ')
disp(' ')
bnorm = norm(f);
i = 0;
x = 0*f;
r = f;
s = el'\(el\f);
z = s;
while (norm(r)/bnorm) > reps & i < 1000
  i = i+1;
  rnormold = r'*z;
  alfa = rnormold/(s'*(a*s));
  x = x+alfa*s;
  r = r-alfa*a*s;
  z = el'\(el\r);
  beta = r'*z/rnormold;
  s = z+beta*s;
  res(i) = norm(r);
  disp([i,norm(r)])
end
for j = 1 : (i-1)
   fac(j) = res(j+1)/res(j);
end
res = log10(res);
subplot(2,1,1);
plot(res);
title(' The iterations using Conjugate Gradients')
xlabel(' iterations')
ylabel(' 10log(residu)')  
subplot(2,1,2);
plot(fac)
title(' The reduction factor |r_(i+1)|/|r_i)|')
xlabel(' iterations')
ylabel(' factor')
